@@ -3,7 +3,7 @@
 #'
 #'   Utilities for checking/handling arguments
 #'
-#'  $Revision: 1.13 $  $Date: 2024/06/05 08:16:37 $
+#'  $Revision: 1.16 $  $Date: 2025/05/08 04:21:49 $
 #'
 
 "%orifnull%" <- function(a, b) {
@@ -295,6 +295,19 @@ explain.ifnot <- function(expr, context="") {
   }
 }
 
+warn.reset.arg <- function(x, xset, context="") {
+  ## force argument 'x' to have value 'xset', with a warning if it was changed.
+  if(!identical(x, xset)) {
+    argname <- deparse(substitute(x))
+    warning(paste("Argument", sQuote(argname),
+                  "was reset to", sQuote(paste(xset)),
+                  context),
+            call.=FALSE)
+  }
+  return(xset)
+}
+
+
 warn.ignored.args <- function(..., context=NULL) {
   if((narg <- length(list(...))) > 0) {
     whinge <- paste(narg, "unrecognised",
